@@ -184,7 +184,7 @@ void ShadowRoot::Unbind() {
     OwnerDoc()->RemoveComposedDocShadowRoot(*this);
   }
 
-  UnbindContext context(*this);
+  UnbindContext context(*this, /* aBatchState = */ nullptr);
   for (nsIContent* child = GetFirstChild(); child;
        child = child->GetNextSibling()) {
     child->UnbindFromTree(context);
@@ -259,18 +259,17 @@ void ShadowRoot::AddSlot(HTMLSlotElement* aSlot) {
 
   InvalidateStyleAndLayoutOnSubtree(aSlot);
 
-  HTMLSlotElement* oldSlot = currentSlots->SafeElementAt(1);
+  HTMLSlotElement* oldSlot = currentSlots.SafeElementAt(1, nullptr);
   if (SlotAssignment() == SlotAssignmentMode::Named) {
     if (oldSlot) {
       MOZ_DIAGNOSTIC_ASSERT(oldSlot != aSlot);
 
       // Move assigned nodes from old slot to new slot.
       InvalidateStyleAndLayoutOnSubtree(oldSlot);
-      const nsTArray<RefPtr<nsINode>>& assignedNodes = oldSlot->AssignedNodes();
       bool doEnqueueSlotChange = false;
-      while (assignedNodes.Length() > 0) {
-        nsINode* assignedNode = assignedNodes[0];
-
+      auto assignedNodes =
+          ToTArray<AutoTArray<nsINode*, 8>>(oldSlot->AssignedNodes());
+      for (nsINode* assignedNode : assignedNodes) {
         oldSlot->RemoveAssignedNode(*assignedNode->AsContent());
         aSlot->AppendAssignedNode(*assignedNode->AsContent());
         doEnqueueSlotChange = true;
@@ -327,11 +326,11 @@ void ShadowRoot::RemoveSlot(HTMLSlotElement* aSlot) {
   MOZ_ASSERT(mSlotMap.Get(name));
 
   SlotArray& currentSlots = *mSlotMap.Get(name);
-  MOZ_DIAGNOSTIC_ASSERT(currentSlots->Contains(aSlot),
+  MOZ_DIAGNOSTIC_ASSERT(currentSlots.Contains(aSlot),
                         "Slot to de-register wasn't found?");
-  if (currentSlots->Length() == 1) {
+  if (currentSlots.Length() == 1) {
     MOZ_ASSERT_IF(SlotAssignment() == SlotAssignmentMode::Named,
-                  currentSlots->ElementAt(0) == aSlot);
+                  currentSlots.ElementAt(0) == aSlot);
 
     InvalidateStyleAndLayoutOnSubtree(aSlot);
 
@@ -351,7 +350,7 @@ void ShadowRoot::RemoveSlot(HTMLSlotElement* aSlot) {
     }
   }
 
-  const bool wasFirstSlot = currentSlots->ElementAt(0) == aSlot;
+  const bool wasFirstSlot = currentSlots.ElementAt(0) == aSlot;
   currentSlots.RemoveElement(*aSlot);
   if (!wasFirstSlot || SlotAssignment() == SlotAssignmentMode::Manual) {
     return;
@@ -360,16 +359,15 @@ void ShadowRoot::RemoveSlot(HTMLSlotElement* aSlot) {
   // Move assigned nodes from removed slot to the next slot in
   // tree order with the same name.
   InvalidateStyleAndLayoutOnSubtree(aSlot);
-  HTMLSlotElement* replacementSlot = currentSlots->ElementAt(0);
-  const nsTArray<RefPtr<nsINode>>& assignedNodes = aSlot->AssignedNodes();
+  HTMLSlotElement* replacementSlot = currentSlots.ElementAt(0);
+  auto assignedNodes =
+      ToTArray<AutoTArray<nsINode*, 8>>(aSlot->AssignedNodes());
   if (assignedNodes.IsEmpty()) {
     return;
   }
 
   InvalidateStyleAndLayoutOnSubtree(replacementSlot);
-  while (!assignedNodes.IsEmpty()) {
-    nsINode* assignedNode = assignedNodes[0];
-
+  for (auto* assignedNode : assignedNodes) {
     aSlot->RemoveAssignedNode(*assignedNode->AsContent());
     replacementSlot->AppendAssignedNode(*assignedNode->AsContent());
   }
@@ -659,7 +657,7 @@ ShadowRoot::SlotInsertionPoint ShadowRoot::SlotInsertionPointFor(
     if (!slots) {
       return {};
     }
-    slot = (*slots)->ElementAt(0);
+    slot = (*slots).ElementAt(0);
   }
 
   MOZ_ASSERT(slot);
@@ -691,7 +689,7 @@ ShadowRoot::SlotInsertionPoint ShadowRoot::SlotInsertionPointFor(
       return {slot, Some(index)};
     }
   } else {
-    const nsTArray<RefPtr<nsINode>>& assignedNodes = slot->AssignedNodes();
+    const Span assignedNodes = slot->AssignedNodes();
     nsIContent* currentContent = GetHost()->GetFirstChild();
     for (uint32_t i = 0; i < assignedNodes.Length(); i++) {
       // Seek through the host's explicit children until the
@@ -763,11 +761,13 @@ void ShadowRoot::MaybeReassignMainSummary(SummaryChangeReason aReason) {
   if (aReason == SummaryChangeReason::Insertion) {
     // We've inserted a summary element, may need to remove the existing one.
     SlotArray* array = mSlotMap.Get(u"internal-main-summary"_ns);
-    MOZ_RELEASE_ASSERT(array && (*array)->Length() == 1);
-    HTMLSlotElement* slot = (*array)->ElementAt(0);
-    auto* summary = HTMLSummaryElement::FromNodeOrNull(
-        slot->AssignedNodes().SafeElementAt(0));
-    if (summary) {
+    MOZ_RELEASE_ASSERT(array && (*array).Length() == 1);
+    HTMLSlotElement* slot = (*array).ElementAt(0);
+    auto assigned = slot->AssignedNodes();
+    if (assigned.IsEmpty()) {
+      return;
+    }
+    if (auto* summary = HTMLSummaryElement::FromNode(assigned[0])) {
       MaybeReassignContent(*summary);
     }
   } else if (MOZ_LIKELY(GetHost())) {
@@ -841,9 +841,8 @@ void ShadowRoot::MaybeUnslotHostChild(nsIContent& aChild) {
     InvalidateStyleAndLayoutOnSubtree(slot);
   }
 
-  slot->RemoveAssignedNode(aChild);
   slot->EnqueueSlotChangeEvent();
-
+  slot->RemoveAssignedNode(aChild);
   if (mIsDetailsShadowTree && aChild.IsHTMLElement(nsGkAtoms::summary)) {
     MaybeReassignMainSummary(SummaryChangeReason::Deletion);
   }
