@@ -121,12 +121,17 @@ static bool ShouldHaveDirectoryService() {
   return GeckoProcessType_Default == XRE_GetProcessType();
 }
 
+#ifdef XP_IOS
+// Hack to ensure environment variables are copied into child processes on iOS.
+extern char** environ;
+#endif
+
 namespace mozilla {
 namespace ipc {
 
 struct LaunchResults {
   base::ProcessHandle mHandle = 0;
-#ifdef XP_DARWIN
+#ifdef XP_MACOSX
   task_t mChildTask = MACH_PORT_NULL;
 #endif
 #ifdef XP_IOS
@@ -345,6 +350,7 @@ class IosProcessLauncher : public PosixProcessLauncher {
       : PosixProcessLauncher(aHost, std::move(aExtraOpts)) {}
 
  protected:
+  virtual Result<Ok, LaunchError> DoSetup() override;
   virtual RefPtr<ProcessLaunchPromise> DoLaunch() override;
 
   DarwinObjectPtr<xpc_object_t> mBootstrapMessage;
@@ -382,7 +388,7 @@ GeckoChildProcessHost::GeckoChildProcessHost(GeckoProcessType aProcessType,
 #endif
       mHandleLock("mozilla.ipc.GeckoChildProcessHost.mHandleLock"),
       mChildProcessHandle(0),
-#if defined(XP_DARWIN)
+#if defined(XP_MACOSX)
       mChildTask(MACH_PORT_NULL),
 #endif
 #if defined(MOZ_SANDBOX) && defined(XP_MACOSX)
@@ -419,7 +425,7 @@ GeckoChildProcessHost::~GeckoChildProcessHost() {
 
   {
     mozilla::AutoWriteLock hLock(mHandleLock);
-#if defined(XP_DARWIN)
+#if defined(XP_MACOSX)
     if (mChildTask != MACH_PORT_NULL) {
       mach_port_deallocate(mach_task_self(), mChildTask);
     }
@@ -456,7 +462,7 @@ base::ProcessId GeckoChildProcessHost::GetChildProcessId() {
   return base::GetProcId(mChildProcessHandle);
 }
 
-#ifdef XP_DARWIN
+#ifdef XP_MACOSX
 task_t GeckoChildProcessHost::GetChildTask() {
   mozilla::AutoReadLock handleLock(mHandleLock);
   return mChildTask;
@@ -770,7 +776,7 @@ bool GeckoChildProcessHost::AsyncLaunch(
                     // "safe" invalid value to use in places like this.
                     aResults.mHandle = 0;
 
-#ifdef XP_DARWIN
+#ifdef XP_MACOSX
                     this->mChildTask = aResults.mChildTask;
 #endif
 #ifdef XP_IOS
@@ -783,7 +789,7 @@ bool GeckoChildProcessHost::AsyncLaunch(
                     if (mNodeChannel) {
                       mNodeChannel->SetOtherPid(
                           base::GetProcId(this->mChildProcessHandle));
-#ifdef XP_DARWIN
+#ifdef XP_MACOSX
                       mNodeChannel->SetMachTaskPort(this->mChildTask);
 #endif
                     }
@@ -994,7 +1000,7 @@ IPCLaunchThreadObserver::Observe(nsISupports* aSubject, const char* aTopic,
   return rv;
 }
 
-nsCOMPtr<nsIEventTarget> GetIPCLauncher() {
+nsCOMPtr<nsISerialEventTarget> GetIPCLauncher() {
   StaticMutexAutoLock lock(gIPCLaunchThreadMutex);
   if (!gIPCLaunchThread) {
     nsCOMPtr<nsIThread> thread;
@@ -1011,7 +1017,7 @@ nsCOMPtr<nsIEventTarget> GetIPCLauncher() {
     }
   }
 
-  nsCOMPtr<nsIEventTarget> thread = gIPCLaunchThread.get();
+  nsCOMPtr<nsISerialEventTarget> thread = gIPCLaunchThread.get();
   MOZ_DIAGNOSTIC_ASSERT(thread);
   return thread;
 }
@@ -1357,6 +1363,38 @@ RefPtr<ProcessLaunchPromise> PosixProcessLauncher::DoLaunch() {
 #endif  // XP_UNIX
 
 #ifdef XP_IOS
+Result<Ok, LaunchError> IosProcessLauncher::DoSetup() {
+  Result<Ok, LaunchError> aError = PosixProcessLauncher::DoSetup();
+  if (aError.isErr()) {
+    return aError;
+  }
+
+  // Unlike on other platforms, the child process will not inherit the parent
+  // process's environment, so we need to manually copy over environment
+  // variables.
+  for (char** envp = environ; *envp != 0; ++envp) {
+    // Only copy over `MOZ_` keys.
+    if (strncmp(*envp, "MOZ_", 4) != 0) {
+      continue;
+    }
+
+    std::string_view env{*envp};
+    size_t eqIdx = env.find('=');
+    if (eqIdx == std::string_view::npos) {
+      continue;
+    }
+
+    std::string key{env.substr(0, eqIdx)};
+    if (mLaunchOptions->env_map.count(key)) {
+      // If the key already exists in the map, we don't want to overwrite it.
+      continue;
+    }
+
+    mLaunchOptions->env_map[key] = env.substr(eqIdx + 1);
+  }
+  return Ok();
+}
+
 RefPtr<ProcessLaunchPromise> IosProcessLauncher::DoLaunch() {
   ExtensionKitProcess::Kind kind = ExtensionKitProcess::Kind::WebContent;
   if (mProcessType == GeckoProcessType_GPU) {
@@ -1473,27 +1511,16 @@ RefPtr<ProcessLaunchPromise> IosProcessLauncher::DoLaunch() {
             return;
           }
 
-          // FIXME: We have to trust the child to tell us its pid & mach task.
+          // NOTE: We have to trust the child to tell us its pid. The child is
+          // blocked by sandbox from sending its mach task.
           // WebKit uses `xpc_connection_get_pid` to get the pid, however this
           // is marked as unavailable on iOS.
           //
-          // Given how the process is started, however, validating this
-          // information it sends us this early during startup may be
-          // unnecessary.
-          self->mResults.mChildTask =
-              xpc_dictionary_copy_mach_send(reply, "task");
+          // The process hasn't had a chance to load any untrusted content at
+          // this point, so we should be able to trust it.
           pid_t pid =
               static_cast<pid_t>(xpc_dictionary_get_int64(reply, "pid"));
-          CHROMIUM_LOG(INFO) << "ExtensionKit process started, task: "
-                             << self->mResults.mChildTask << ", pid: " << pid;
-
-          pid_t taskPid;
-          kern_return_t kr = pid_for_task(self->mResults.mChildTask, &taskPid);
-          if (kr != KERN_SUCCESS || pid != taskPid) {
-            CHROMIUM_LOG(ERROR) << "Could not validate child task matches pid";
-            promise->Reject(LaunchError("pid_for_task mismatch"), __func__);
-            return;
-          }
+          CHROMIUM_LOG(INFO) << "ExtensionKit process started, pid: " << pid;
 
           self->mResults.mHandle = pid;
           promise->Resolve(std::move(self->mResults), __func__);
