@@ -47,6 +47,7 @@
 #include "vm/Time.h"
 #include "vm/WrapperObject.h"
 
+#include "gc/AtomMarking-inl.h"
 #include "gc/PrivateIterators-inl.h"
 #include "vm/GeckoProfiler-inl.h"
 #include "vm/JSObject-inl.h"
@@ -665,6 +666,8 @@ IncrementalProgress GCRuntime::markWeakReferences(
     }
   }
 
+  markIncomingSymbolEdgesFromUncollectedZones();
+
   bool markedAny = true;
   while (markedAny) {
     if (!marker().markUntilBudgetExhausted(budget)) {
@@ -689,6 +692,33 @@ IncrementalProgress GCRuntime::markWeakReferences(
   return Finished;
 }
 
+void GCRuntime::markIncomingSymbolEdgesFromUncollectedZones() {
+  // We need to mark ephemeron edges where the source is a live symbol that is
+  // referenced from an uncollected zone and which may not have been marked in
+  // this GC. At the same time we want to avoid unnecessarily holding on to
+  // symbols in zones GCs (by marking them as referenced in the atom marking
+  // bitmap), which is why we don't just mark all such symbols at the start of
+  // GC.
+  //
+  // Atoms referenced by uncollected zones will be marked later in
+  // updateAtomsBitmap() which prevents them dying, but since this is after
+  // we've done ephemeron marking it won't mark through the ephemeron edges.
+
+  if (!atomsZone()->isGCMarking()) {
+    return;
+  }
+
+  for (auto iter = atomsZone()->gcEphemeronEdges().iter(); !iter.done();
+       iter.next()) {
+    auto* symbol = iter.get().key()->as<JS::Symbol>();
+    if (isSymbolReferencedByUncollectedZone(symbol, marker().markColor())) {
+      TraceManuallyBarrieredEdge(marker().tracer(), &symbol,
+                                 "incoming symbol edge");
+      MOZ_ASSERT(symbol == iter.get().key());
+    }
+  }
+}
+
 IncrementalProgress GCRuntime::markWeakReferencesInCurrentGroup(
     SliceBudget& budget) {
   return markWeakReferences<SweepGroupZonesIter>(budget);
@@ -1391,12 +1421,14 @@ void GCRuntime::sweepMisc() {
 }
 
 void GCRuntime::sweepCompressionTasks() {
-  JSRuntime* runtime = rt;
+  // Discard pending compression entries for ScriptSources that have no
+  // other references.
+  rt->pendingCompressions().eraseIf(
+      [&](const auto& entry) { return entry.shouldCancel(); });
 
   // Attach finished compression tasks.
   AutoLockHelperThreadState lock;
-  AttachFinishedCompressions(runtime, lock);
-  SweepPendingCompressions(lock);
+  AttachFinishedCompressions(rt, lock);
 }
 
 void GCRuntime::sweepWeakMaps() {
