@@ -177,7 +177,8 @@ nsMathMLmencloseFrame::InheritAutomaticData(nsIFrame* aParent) {
   // let the base class get the default from our parent
   nsMathMLContainerFrame::InheritAutomaticData(aParent);
 
-  mPresentationData.flags |= NS_MATHML_STRETCH_ALL_CHILDREN_VERTICALLY;
+  mPresentationData.flags +=
+      MathMLPresentationFlag::StretchAllChildrenVertically;
 
   InitNotations();
 
@@ -267,9 +268,9 @@ void nsMathMLmencloseFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
 }
 
 /* virtual */
-nsresult nsMathMLmencloseFrame::Place(DrawTarget* aDrawTarget,
-                                      const PlaceFlags& aFlags,
-                                      ReflowOutput& aDesiredSize) {
+void nsMathMLmencloseFrame::Place(DrawTarget* aDrawTarget,
+                                  const PlaceFlags& aFlags,
+                                  ReflowOutput& aDesiredSize) {
   ///////////////
   // Measure the size of our content using the base class to format like an
   // inferred mrow, without border/padding.
@@ -277,12 +278,7 @@ nsresult nsMathMLmencloseFrame::Place(DrawTarget* aDrawTarget,
   PlaceFlags flags = aFlags + PlaceFlag::MeasureOnly +
                      PlaceFlag::IgnoreBorderPadding +
                      PlaceFlag::DoNotAdjustForWidthAndHeight;
-  nsresult rv = nsMathMLContainerFrame::Place(aDrawTarget, flags, baseSize);
-
-  if (NS_FAILED(rv)) {
-    DidReflowChildren(PrincipalChildList().FirstChild());
-    return rv;
-  }
+  nsMathMLContainerFrame::Place(aDrawTarget, flags, baseSize);
 
   nsBoundingMetrics bmBase = baseSize.mBoundingMetrics;
   nscoord dx_left = 0, dx_right = 0;
@@ -571,8 +567,6 @@ nsresult nsMathMLmencloseFrame::Place(DrawTarget* aDrawTarget,
     PositionRowChildFrames(dx_left + borderPadding.left,
                            aDesiredSize.BlockStartAscent());
   }
-
-  return NS_OK;
 }
 
 nscoord nsMathMLmencloseFrame::FixInterFrameSpacing(
