@@ -247,8 +247,9 @@ struct nsCSSRendering {
    * Uses a fixed style equivalent to "1px dotted |aColor|".
    * Not used for controls, because the native theme may differ.
    */
-  static void PaintFocus(nsPresContext* aPresContext, DrawTarget* aDrawTarget,
-                         const nsRect& aFocusRect, nscolor aColor);
+  static nsCSSBorderRenderer GetBorderRendererForFocus(nsIFrame*, DrawTarget*,
+                                                       const nsRect& aFocusRect,
+                                                       nscolor aColor);
 
   /**
    * Render a gradient for an element.
@@ -554,9 +555,16 @@ struct nsCSSRendering {
       mozilla::Side aStartBevelSide, nscoord aStartBevelOffset,
       mozilla::Side aEndBevelSide, nscoord aEndBevelOffset);
 
-  // NOTE: pt, dirtyRect, lineSize, ascent, offset in the following
-  //       structs are non-rounded device pixels, not app units.
+  // NOTE: pt, dirtyRect, lineSize, ascent, offset, insetLeft, and insetRight in
+  //       the following structs are non-rounded device pixels, not app units.
   struct DecorationRectParams {
+    // Checks if either start or end inset value is negative.
+    // In that situation, text decoration will extend past the edges of the
+    // text.
+    bool HasNegativeInset() const {
+      return insetLeft < 0.0 || insetRight < 0.0;
+    }
+
     // The width [length] and the height [thickness] of the decoration
     // line. This is a "logical" size in textRun orientation, so that
     // for a vertical textrun, width will actually be a physical height;
@@ -589,10 +597,15 @@ struct nsCSSRendering {
     // The style of the decoration line
     mozilla::StyleTextDecorationStyle style =
         mozilla::StyleTextDecorationStyle::None;
+    // Decoration-line start/end inset values for this decoration.
+    // The caller must ensure that the start/end values are translated to
+    // left/right in the line-relative direction.
+    Float insetLeft = 0.0f;
+    Float insetRight = 0.0f;
     bool vertical = false;
     bool sidewaysLeft = false;
     gfxTextRun::Range glyphRange;
-    gfxTextRun::PropertyProvider* provider;
+    gfxTextRun::PropertyProvider* provider = nullptr;
   };
 
   struct PaintDecorationLineParams : DecorationRectParams {
@@ -610,6 +623,9 @@ struct nsCSSRendering {
     Float baselineOffset = 0.0f;
     // Whether text-decoration-skip-ink behavior is to be supported.
     bool allowInkSkipping = true;
+
+    mozilla::StyleTextDecorationSkipInk skipInk =
+        mozilla::StyleTextDecorationSkipInk::None;
   };
 
   /**
@@ -750,7 +766,7 @@ struct nsCSSRendering {
  * nsContextBoxBlur
  * Creates an 8-bit alpha channel context for callers to draw in, blurs the
  * contents of that context and applies it as a 1-color mask on a
- * different existing context. Uses gfxAlphaBoxBlur as its back end.
+ * different existing context. Uses gfxGaussianBlur as its back end.
  *
  * You must call Init() first to create a suitable temporary surface to draw
  * on.  You must then draw any desired content onto the given context, then
@@ -765,7 +781,7 @@ class nsContextBoxBlur {
   typedef mozilla::gfx::RectCornerRadii RectCornerRadii;
 
  public:
-  enum { FORCE_MASK = 0x01, DISABLE_HARDWARE_ACCELERATION_BLUR = 0x02 };
+  enum { FORCE_MASK = 0x01 };
   /**
    * Prepares a gfxContext to draw on. Do not call this twice; if you want
    * to get the gfxContext again use GetContext().
@@ -911,7 +927,7 @@ class nsContextBoxBlur {
                                      mozilla::gfx::IntSize& aOutSpreadRadius,
                                      bool aConstrainSpreadRadius = true);
 
-  gfxAlphaBoxBlur mAlphaBoxBlur;
+  gfxGaussianBlur mGaussianBlur;
   mozilla::UniquePtr<gfxContext> mOwnedContext;
   gfxContext* mContext;  // may be either mOwnedContext or mDestinationContext
   gfxContext* mDestinationCtx;
