@@ -7,6 +7,7 @@
 //! [scope]: https://drafts.csswg.org/css-cascade-6/#scoped-styles
 
 use crate::applicable_declarations::ScopeProximity;
+use crate::derives::*;
 use crate::dom::TElement;
 use crate::parser::ParserContext;
 use crate::selector_parser::{SelectorImpl, SelectorParser};
@@ -14,7 +15,6 @@ use crate::shared_lock::{
     DeepCloneWithLock, Locked, SharedRwLock, SharedRwLockReadGuard, ToCssWithGuard,
 };
 use crate::simple_buckets_map::SimpleBucketsMap;
-use crate::str::CssStringWriter;
 use crate::stylesheets::CssRules;
 use cssparser::{Parser, SourceLocation, ToCss};
 #[cfg(feature = "gecko")]
@@ -27,7 +27,7 @@ use selectors::parser::{Component, ParseRelative, Selector, SelectorList};
 use selectors::OpaqueElement;
 use servo_arc::Arc;
 use std::fmt::{self, Write};
-use style_traits::{CssWriter, ParseError};
+use style_traits::{CssStringWriter, CssWriter, ParseError};
 
 /// A scoped rule.
 #[derive(Debug, ToShmem)]
@@ -128,10 +128,6 @@ fn parse_scope<'a>(
             return Ok(None);
         }
         input.parse_nested_block(|input| {
-            if input.is_exhausted() {
-                // `@scope () {}` is valid.
-                return Ok(None);
-            }
             let selector_parser = SelectorParser {
                 stylesheet_origin: context.stylesheet_origin,
                 namespaces: &context.namespaces,
@@ -307,9 +303,6 @@ where
     let mut parent = Some(element);
     let mut proximity = 0usize;
     while let Some(p) = parent {
-        if ceiling == Some(p.opaque()) {
-            break;
-        }
         if target.check(p, ceiling, scope_subject_map, context) {
             result.push(ScopeRootCandidate {
                 root: p.opaque(),
@@ -318,6 +311,9 @@ where
             // Note that we can't really break here - we need to consider
             // ALL scope roots to figure out whch one didn't end.
         }
+        if ceiling == Some(p.opaque()) {
+            break;
+        }
         parent = p.parent_element();
         proximity += 1;
         // We we got to the top of the shadow tree - keep going
