@@ -3,6 +3,7 @@
  * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
 
 //! Generic types for CSS values related to flexbox.
+use crate::derives::*;
 
 /// A generic value for the `flex-basis` property.
 #[derive(
@@ -12,7 +13,6 @@
     Copy,
     Debug,
     MallocSizeOf,
-    Parse,
     PartialEq,
     SpecifiedValueInfo,
     ToAnimatedValue,
